#***********************************************************************
# Licensed Material - Property of IBM
#
# 5724-M24
#
# (c) Copyright IBM Corp. 2008.  All rights reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#************************************************************************

#####################################################################################
# Deploys Websphere Business Services Fabric application - Fix pack 6.2.0
# 
# wsadmin <wsadmin options> -f installFix.py  [-node nodeName]
#                                                  [-toolsUserId userId]
#                                                  [-fabricTemplatePath fabricTemplatePath]
#
# If no parameters are provided, all required information will be requested 
# on the command line by the script.
#
# The nodeName is name of the node where fabric enterprise application would be deployed.The first server
# in that node would host the application
# 
# The -toolsUserId value is the userid who would have have access to Fabric Adminstration console.
# The userid needs to a valid user in websphere realm.
#
# -fabricTemplatePath is the WBSFabric profile template directory. 
# The location is WPS_INSTALL_Location\profileTemplates\WBSFabric.The script picks up the fabric
# enterprise application from profile template directory and deploys it on cluster.
#  
#############################################################


import sys
import os


##########################################################
# Prints usage error 
###########################################################
def usageError(error):
    print ""
    print "*************************************************************"
    print "Incorrect usage -" + error
    print "*************************************************************"
    print ""
    print "wsadmin <wsadmin options> -f installFix.py  [-server serverName]"
    print "                                              [-toolsUserId userId]"
    print "                                              [-fabricTemplatePath fabricTemplatePath]"
    print ""
    print ""
    sys.exit()


###########################################################
# Prints error message on the console
###########################################################
def validationError (errorMessage) :
    print ""
    print "*********************************************"
    print "Error -"+errorMessage
    print "*********************************************"
    print ""
    sys.exit()

##################################################################
# Returns the name of the first node in the profile.
##################################################################

def fetchFirstNodeName():
    nodeList = AdminConfig.list('Node')
    nodeList = nodeList.split(lineSeparator)
    defaultNodeIndex = 0
    if(len(nodeList) > 0):
        
        # Printing Node List
        
        print "<---------------- Following nodes are available in profile ----------->"  
        i = 1
        for node in nodeList:
            print i,")",AdminConfig.showAttribute(node, 'name')
            i = i + 1
        
        print AdminConfig.showAttribute(nodeList[0],'name'), " has been selected" 
        return AdminConfig.showAttribute(nodeList[0],'name')
    
    else:
        usageError("specify the node name")
           
############################################################
#Get Cluster names in the cell.
############################################################
def getClusterNames(cell) :
    clusterlists = AdminConfig.list("ServerCluster", cell).splitlines()
    clusterList = []
    for clid in clusterlists:
	i = clid.find('(')
        name = clid[0:i]
        clusterList.append(name)
    return clusterList

############################################################
#Get Nodes names in the cell.
############################################################
def getNodeNames(cell) :
    nodes = AdminConfig.list("Node", cell).splitlines()
    nodeList = []
    for node in nodes:
        i = node.find('(')
        name = node[0:i]
        nodeList.append(name)
    return nodeList
        
############################################################
# Variables
############################################################

#Conditional variables
True = 1 == 1
False = 1 == 0
interactive = False
doCluster = False
newLine = java.lang.System.getProperty("line.separator");

userId = ""
password = ""
cellType = ""
cellName = ""
cellId = ""
nodeId = ""
serverId = ""
nodeName = ""
serverName = ""
dmgrPath = ""
fabricBusAuthAlias = ""
fabricTemplatePath = ""

#EAR Path
catalogEarPath = ""
toolsEARPath = ""
engineEARPath = ""
toolsHelpEARPath = ""
servicesEARPath = ""

###################################################################
# Check if the script is being run interactively
###################################################################
if ((len(sys.argv) == 0)):
	interactive = True
        
###################################################################
# Command line parameters passed in. Evaluate the parameters
###################################################################
else:
    nextArg = 0;
    while(len(sys.argv) > nextArg) :
        param = sys.argv[nextArg]
        nextArg += 1

        if (len(sys.argv) <= nextArg):
            usageError("specify value for" + param + " parameter")
        value = sys.argv[nextArg]
        nextArg += 1
   
        if(param == "-cell"):
            cellName = value 
        elif(param == "-node"):
            nodeName = value  
        elif (param == "-toolsUserId"):
            userId = value
        elif (param == "-fabricTemplatePath"):
            fabricTemplatePath = value.strip().replace("\\", "/")
        else :
            usageError("Unknown parameter " + param)

###################################################################
# Validate paramter values
###################################################################
    cells = AdminConfig.list("Cell").split(lineSeparator)
    if(len(cells) > 1):
        if(len(cellName) < 1):
            usageError("specify the cell name")
    else :
        try :
            cellId = cells[0]
        except :
            print "The profile doesn't have any cell configured."

        i = cellId.find('(')
        cellName = cellId[0:i]

        if(cellId == ""):
	        validationError("cell " + cellName + " doesn't exist")

      
    # Fetch the first node name if node name is not provided. 
    if(len(nodeName) < 1):
        nodeName = fetchFirstNodeName()                    
    
    nodeId = AdminConfig.getid("/Node:" + nodeName + "/")    
    if(nodeId == ""):
	        validationError("Node " + nodeName + " doesn't exist.")
	    

    #Get the First Server in the List.
    servers = AdminConfig.list("Server", nodeId)
    serversList = servers.split(lineSeparator)
    if(len(serversList) < 1):
       validationError("No Server configured for Node " + nodeName)
	
    serverName = AdminConfig.showAttribute(serversList[0], "name")
    serverId = AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/")
    if (serverId == ""):
        validationError("Server " + serverName + " doesn't exist")
    
    if (len(userId) < 1) :
	    usageError("Specify the user id for access to Fabric Admin console.")
 
    if (len(fabricTemplatePath) < 1) :
	    usageError("Specify the directory of the WBSFabric profile template.")
   
  
###################################################################
# End of validation
###################################################################

###################################################################
# If script is being run interactively, query for values.
###################################################################

if (interactive):
    #Get the Cells
    cells = AdminConfig.list("Cell").split(lineSeparator)
    if(len(cells) > 1):
        cellName = raw_input("Enter the cell name. ")
        cellId = AdminConfig.getid("/Cell:" + cellName + "/")
        if(cellId == ""):
            validationError("cell " + cellName + " doesn't exist.")
    else :
        # There is only one cell. Don't ask for cellname.
        try :
            cellId = cells[0]
        except :
            print "The profile doesn't have any cell configured."

        i = cellId.find('(')
        cellName = cellId[0:i]

        if(cellId == ""):
	        validationError("cell " + cellName + " doesn't exist")
   
    nodeList = getNodeNames(cellId)

    if len(nodeList) < 1 :
        validationError("No Nodes configured.")
    elif len(nodeList) > 1 :
        print "The following Nodes are configured"
        cnt = 0
        for clid in nodeList:
            cnt = cnt + 1
            print (str(cnt) + "->" + clid)
            
	    input = int(raw_input("Select the Node to install Fabric application.Enter the number."))
	
        #Validate Input.
        if (len(nodeList) < input):
	        validationError("Wrong input for the Node name.")
        else:
            nodeName = nodeList[input - 1] 
            nodeId = AdminConfig.getid("/Node:" + nodeName + "/")
            if (nodeId == ""):
                validationError("Node " + nodeName + " doesn't exist")
    else :
        nodeName = nodeList[0]
        nodeId = AdminConfig.getid("/Node:" + nodeName + "/")
        if (nodeId == ""):
            validationError("Node " + nodeName + " doesn't exist")
    
    
    #Get First Server
    servers = AdminConfig.list("Server", nodeId)
    serversList = servers.split(lineSeparator)
    if(len(serversList) < 1):
       validationError("No Server configured for Node " + nodeName)
	
    serverName = AdminConfig.showAttribute(serversList[0], "name")
    serverId = AdminConfig.getid("/Node:" + nodeName + "/Server:" + serverName + "/")
    if (serverId == ""):
        validationError("Server " + serverName + " doesn't exist")
    
    
    #Ask for userid and password for Fabric Adminstartion console access
    userId = raw_input("Enter the userid that would have access to Fabric Adminstration console. ")
    if (len(userId) < 1) :
	    usageError("Specify the user id for access to Fabric Administration console")
    
    
    #Ask for profile path
    fabricTemplatePath = raw_input("Enter the directory of the WBSFabric profile template.")
    
    if(fabricTemplatePath == ""):
        validationError("Profile template path for WBSFabric not provided.")
    
    fabricTemplatePath = fabricTemplatePath.strip().replace("\\", "/")

##########################################################################
# Interactive Ends
###########################################################################

##########################################################################
# Common validation for Interactive and Script based.
##########################################################################

profilePath = fabricTemplatePath + "/actions/scripts/installation/runtime/"

catalogEarPath = profilePath + "catalog.ear"

if (os.path.exists(catalogEarPath) == 0) :
    validationError("Cannot find catalog.ear file. Make sure the WBSFabric path provided is correct.")        

toolsEARPath = profilePath + "tools.ear"
if (os.path.exists(toolsEARPath) == 0) :
    validationError("Cannot find tools.ear file. Make sure the WBSFabric path provided is correct.")        


engineEARPath = profilePath + "engine.ear"
if (os.path.exists(engineEARPath) == 0) :
    validationError("Cannot find engine.ear file. Make sure the WBSFabric path provided is correct.")        

toolsHelpEARPath = profilePath + "tools-help.ear"
if (os.path.exists(toolsHelpEARPath) == 0) :
    validationError("Cannot find tools-help.ear. Make sure the WBSFabric path provided is correct.") 

servicesEARPath = profilePath + "services.ear"
if (os.path.exists(servicesEARPath) == 0) :
    validationError("Cannot find services.ear. Make sure the WBSFabric path provided is correct.") 

#######################################################################
#Validation complete

busName = "Fabric." + cellName + ".Bus"

existingFabricBusId = AdminConfig.getid("/SIBus:" + busName + "/")
if (existingFabricBusId == ""):
    validationError("The profile is not configured with Fabric.")
else :
    fabricBusAuthAlias = AdminConfig.showAttribute(existingFabricBusId, "interEngineAuthAlias")


##Deploy the Fabric application

isCatalogError = False
isToolsHelpError = False
isToolsError = False
isEngineError = False
isRESTError = False


#Fabric Engine Starts

try :
    #Check if Application exists
    AdminApp.view('Fabric_Engine')
except :
    validationError("The profile is not configured with Fabric.Fabric_Engine enterprise application doesn't exist in the given profile.")


try :
    #Try to uninstall Catalog
    AdminApp.uninstall('Fabric_Engine')
except :
    validationError("Fabric_Engine enterprise application cannot be uninstalled")
    isEngineError = True
    
if(isEngineError == False):

    try :
        appOptions = ["-appname", "Fabric_Engine", "-server", serverName]
        AdminApp.install(engineEARPath, appOptions)
        print "Deployed Engine ear."
    except :
        validationError("Fabric_Engine enterprise application cannot be installed.")

#Fabric Engine ends

#Fabric REST Service starts
try :
    #Check if Application exists
    AdminApp.view('Fabric_REST_Services')
except :
    validationError("The profile is not configured with Fabric.Fabric_REST_Services enterprise application doesn't exist in the given profile.")

try :
    AdminApp.uninstall('Fabric_REST_Services')
except :
    validationError("Fabric_REST_Services enterprise application cannot be uninstalled.")
    isRESTError = True
    
if(isRESTError == False):
    try :
        appOptions = ["-appname", "Fabric_REST_Services", "-server", serverName]
        AdminApp.install(servicesEARPath, appOptions)
        print "Deployed Fabric_REST_Services ear."
    except :
        validationError("Error - Fabric_REST_Services enterprise application cannot be installed.")

try :
    #Check if Application exists
    AdminApp.view('Fabric_Catalog')
except :
    validationError("The profile is not configured with Fabric.Fabric catalog enterprise application doesn't exist in the given profile.")


try :
    #Try to uninstall Catalog
    AdminApp.uninstall('Fabric_Catalog')
except :
    validationError("Fabric_Catalog enterprise application cannot be uninstalled.")
    isCatalogError = True
    
if(isCatalogError == False):
    try :
        #Install Fabric Catalog Ear File
        appOptions = ["-appname", "Fabric_Catalog", "-server", serverName]
        AdminApp.install(catalogEarPath, appOptions)
        print "Deployed catalog ear."
    except :
        validationError("Error - Fabric_Catalog enterprise application cannot be installed.")

#Catlog Ends

#Tools Help
try :
    #Check if Application exists
    AdminApp.view('Fabric_Tools_Help')
except :
    validationError("The profile is not configured with Fabric.Fabric_Tools_Help enterprise application doesn't exist in the given profile.")

try :
    #Try to uninstall Help
    AdminApp.uninstall('Fabric_Tools_Help')
except :
    validationError("Fabric_Tools_Help enterprise application cannot be uninstalled.")
    isToolsHelpError = True
    
if(isToolsHelpError == False):
    try :
        appOptions = ["-appname", "Fabric_Tools_Help", "-server", serverName]
        AdminApp.install(toolsHelpEARPath, appOptions)
        print "Deployed Tools help ear."
    except :
        validationError("Fabric_Tools_Help enterprise application cannot be installed.")

#Tools Help Ear ends

#Fabric Tools Start

try :
    #Check if Application exists
    AdminApp.view('Fabric_Tools')
except :
    validationError("The profile is not configured with Fabric.Fabric_Tools enterprise application doesn't exist in the given profile.")

try :
    #Try to uninstall Fabric_Tools
    AdminApp.uninstall('Fabric_Tools')
except :
    validationError("Fabric_Tools enterprise application cannot be uninstalled.")
    isToolsError = True
    
if(isToolsError == False):

    try :
        nameOpt = ["-appname", "Fabric_Tools"]
        serverOpt = ["-server", serverName]
        mapUserValue = [["FabricAdministrator", "No", "No" , userId , ""]]
        mapUserOpt = ["-MapRolesToUsers", mapUserValue] 
        appOptions = []
        appOptions.extend(nameOpt)
        appOptions.extend(mapUserOpt)
        appOptions.extend(serverOpt)
        AdminApp.install(toolsEARPath, appOptions)
        print "Deployed Tools ear."
    except :
        validationError("Fabric_Tools enterprise application cannot be installed.")

#Fabric Tools ends


try :
    AdminConfig.save()
    print "Done with the Fabric iFIX 6.2.0.001 update."
except Exception, e:
    print "An error occured while saving all changes.Please execute the script again."
    print "The exception is " + str(sys.exc_info()[0])




##############################################

